import java.io.*;
import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;

// Classe de Chiffrement
class Chiffre {

	

	public static void main(String[] args) {
		if (args.length !=1) {
			System.out.println("Usage : Chiffrement nomDuFichierASigner");
			return ;
		}
		try {
			// Pour generer, il nous faut une source aleatoire :
			SecureRandom random = SecureRandom.getInstance("SHA1PRNG");

			// On genere d'abord une paire clef publique, clef privee.
			System.out.println("Generation de la paire publique privee");
			
			// Definition de la methode de generation (methode de classe statique)
			KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA");

			int keyLength = 1024;
			// Définition de la taille de clef :
			keyPairGen.initialize(keyLength,random);

			// Generation des clefs :
			KeyPair pair = keyPairGen.generateKeyPair();
			
			// on recupere clef publique et clef privee
			// a partir de la paire.
			PrivateKey priv = pair.getPrivate();
			PublicKey pub = pair.getPublic();

			// On va ensuite generer une clef secrete pour le chiffrement
			// symetrique (ici : Blowfish)
			System.out.println("Generation de la clef secrete");
			KeyGenerator keyGen = KeyGenerator.getInstance("BlowFish");
			int secretKeyLength = 64;
			// Définition de la taille de clef :
			keyGen.init(secretKeyLength,random);

			// Generation de la clef secrete :
			Key secretKey = keyGen.generateKey();

			// On obtient un objet permettant de chiffrer
			// avec une methode donne (ici Blowfish) et un mode operatoire
			// donné (ici ECB).
			System.out.println("Chiffrement avec la clef secrete");
			Cipher cipher = Cipher.getInstance("Blowfish/ECB/PKCS5Padding");

			// On initialise l'objet de chiffrement avec la clef secrete :
			cipher.init(Cipher.ENCRYPT_MODE, secretKey);

			// On récupere ensuite l'ensemble des donnees dans le fichier
			// par paquets et on les passe a l'objet qui chiffre
			byte[] input = Fichier.LireFichierBinaire("document.txt");
			byte[] encrypted = new byte[cipher.getOutputSize(input.length)];

			int ctLength = cipher.update(input, 0, input.length, encrypted, 0);
			ctLength += cipher.doFinal(encrypted, ctLength);

			// Generation du code chiffré
			cipher.doFinal(encrypted);

			// Sauver le fichier chiffre dans un fichier
			System.out.println("Sauvegarde du fichier chiffre ");
			FileOutputStream sigfos = new FileOutputStream("chiffre.bin");
			sigfos.write(encrypted);
			sigfos.close();

			/* ----------------------------------
			 * Ici on chiffre la clef secrete avec la clef publique du recepteur
			 */

			// Sauver la clef secrete
			System.out.println("Chiffrement de la clef secrete");
			
			// On recupere les octets de la clef secrete :
			byte[] key = secretKey.getEncoded();
			System.out.println("Spec :"+secretKey.getFormat());

			// On recupere un objet de chiffrement pour RSA
			cipher = Cipher.getInstance("RSA");

			// On initialise l'objet de chiffrement avec la clef publique
			cipher.init(Cipher.ENCRYPT_MODE, pub);

			// On met a jour l'objet de chiffrement avec les octets
			// de la clef secrete.
			cipher.update(key);
			

			// Generation du code chiffré
			encrypted = cipher.doFinal();

			// Finalement, on fait un wrap pour envelopper la clef

			// Sauvegarde de la clef secrete;
			System.out.println("sauvegarde de la clef secrete");
			FileOutputStream keyfos = new FileOutputStream("secretKey");
			keyfos.write(encrypted);
			keyfos.close();

			
			// Ceci pour recreer la clef secrete
			// a partir des octets de la clef publique precedente.
 			// une keyFactory et un KeySpec

			// On initialise l'objet de chiffrement avec la clef publique
			// On recupere un objet de dechiffrement pour RSA
			cipher = Cipher.getInstance("RSA");
			cipher.init(Cipher.DECRYPT_MODE, priv);

			System.out.println("dechiffrement de la clef secrete");
			cipher.update(encrypted);
			byte[] decrypted = cipher.doFinal();

			// On recree un objet de type clef secrete...
		     	SecretKeySpec decryptedKey = new SecretKeySpec(decrypted,"BlowFish");

			System.out.println("dechiffrement du message");

			cipher = Cipher.getInstance("Blowfish/ECB/PKCS5Padding");

			// On initialise l'objet de chiffrement avec la clef secrete :
			cipher.init(Cipher.DECRYPT_MODE, decryptedKey);

			// On récupere ensuite l'ensemble des donnees dans le fichier
			// par paquets et on les passe a l'objet qui signe.
			// (il maintient a jour l'ensemble des donnees a signer)
			input = Fichier.LireFichierBinaire("chiffre.bin");

			// Ce tableau va contenir la totalite du message dechiffre y compris
			// le padding eventuel (qu'il faudra eliminer)
			byte[] decryptedMessage = new byte[cipher.getOutputSize(input.length)];
			System.out.println("longueur prevue: "+cipher.getOutputSize(input.length));

			// ctLenght contiendra au final la longueur du message dechiffre
			// prive du padding
			ctLength = cipher.update(input, 0, input.length, decryptedMessage, 0);
			System.out.println("longueur : "+ctLength);
			ctLength += cipher.doFinal(decryptedMessage, ctLength);
			System.out.println("longueur : "+ctLength);

			// On recopie le message dechiffre sans le padding
			byte[] plainText = new byte[ctLength];
			System.arraycopy (decryptedMessage, 0, plainText, 0, ctLength);


			
			// Sauver le fichier chiffre dans un fichier
			System.out.println("Sauvegarde du fichier dechiffre ");
			sigfos = new FileOutputStream("dechiffre.bin");
			sigfos.write(plainText);
			sigfos.close();
					
			//System.out.println("clef publique : longueur :"+key.length+" Valeur : "+key.toString());

			
			
		} catch (Exception e) {
			System.out.println("Exception : "+ e.toString());
		}

	return ;
	}
}

