import java.io.*;
import java.security.*;
import java.security.spec.*;

// Classe de generation de signature avec du DSA
class GenSig {

	

	public static void main(String[] args) {
		if (args.length !=1) {
			System.out.println("Usage : GenSig nomDuFichierASigner");
			return ;
		}
		try {
			// Pour generer, il nous faut une source aleatoire :
			SecureRandom random = SecureRandom.getInstance("SHA1PRNG");

			// Definition de la methode de generation (methode de classe statique)
			KeyPairGenerator keyGen = KeyPairGenerator.getInstance("DSA");

			int keyLength = 1024;
			// Définition de la taille de clef :
			keyGen.initialize(keyLength,random);

			// Generation des clefs :
			KeyPair pair = keyGen.generateKeyPair();
			
			// on recupere clef publique et clef privee
			// a partir de la paire.
			PrivateKey priv = pair.getPrivate();
			PublicKey pub = pair.getPublic();

			// On obtient un objet permettant de Signer
			// avec une methode donne (calcul du hash : SHA1 ) et
			// associe a un type de clefs (DSA)
			Signature dsa = Signature.getInstance("SHA1withDSA");

			// On initialise l'objet de chiffrement avec la clef privee :
			dsa.initSign(priv);

			// On récupere ensuite l'ensemble des donnees dans le fichier
			// par paquets et on les passe a l'objet qui signe.
			// (il maintient a jour l'ensemble des donnees a signer)
			FileInputStream fis = new FileInputStream(args[0]);
			BufferedInputStream bufin = new BufferedInputStream(fis);
			byte[]  buffer = new byte[512];

			int len;
			while (bufin.available() != 0){
				len = bufin.read(buffer);
				dsa.update(buffer,0,len);
			}

			bufin.close();

			// Generation de la signature :
			byte[] realSign = dsa.sign();

			System.out.println("Signature : longueur :"+realSign.length+" Valeur : "+realSign.toString());

			// Nous allons pouvoir tester tout ca :
			
			// Sauver la signature dans un fichier
			FileOutputStream sigfos = new FileOutputStream("signature");
			sigfos.write(realSign);
			sigfos.close();

			// Sauver la clef publique
			byte[] key = pub.getEncoded();
			FileOutputStream keyfos = new FileOutputStream("publicKey");
			keyfos.write(key);
			keyfos.close();

			// Ceci pour recreer une clef publique
			// a partir des octets de la clef publique precedente.
 			// une keyFactory et un KeySpec
			EncodedKeySpec pubKeySpec = new X509EncodedKeySpec(key);
			KeyFactory keyFactory = KeyFactory.getInstance("DSA");

			// Avec la specification et la keyFactory on peut construire la PublicKey :
			PublicKey pubKey = keyFactory.generatePublic(pubKeySpec);
			
					       //

			System.out.println("clef publique : longueur :"+key.length+" Valeur : "+key.toString());

			
			
		} catch (Exception e) {
			System.out.println("Exception : "+ e.toString());
		}

	return ;
	}
}

